# Gufw 9.04.2 - http://gufw.tuxfamily.org
# Copyright (C) 2009 Marcos Alvarez Costales
#
# Gufw is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
# 
# Gufw is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Gufw; if not, see http://www.gnu.org/licenses for more
# information.


import commands
from Variable import Variable


class Frontend:

    def __init__(self):
        self.variable = Variable()

       
    # Return status ufw
    def set_status(self, status):
        commands.getstatusoutput(self.variable.get_command(status))
        return status


    # Get status ufw
    def get_status(self):
        l_status = commands.getstatusoutput(self.variable.get_command("status"))
        if l_status[1].find("Status: active") != -1:
            return self.variable.get_constant("enabled")
        else:
            return self.variable.get_constant("disabled")


    # Return status ufw
    def set_default(self, default):
        commands.getstatusoutput(self.variable.get_command(default))
        return default


    # Get status ufw
    def get_default(self):
        l_default = commands.getstatusoutput(self.variable.get_command("search_policy"))
        if l_default[1].find("ACCEPT") != -1:
            return self.variable.get_constant("allow")
        elif l_default[1].find("DROP") != -1:
            return self.variable.get_constant("deny")
        elif l_default[1].find("REJECT") != -1:
            return self.variable.get_constant("reject")


    # Get actual rules (NOT SPLIT BY ACTION)
    def get_rules(self):
        l_rules        = commands.getstatusoutput(self.variable.get_command("status"))
        l_rule_lines   = l_rules[1].split("\n")
        l_return_rules = []
        
        for l_descomponent_rules in l_rule_lines:
            
            if l_descomponent_rules.find(self.variable.get_constant("allow_upper"))  != -1 or \
               l_descomponent_rules.find(self.variable.get_constant("deny_upper"))   != -1 or \
               l_descomponent_rules.find(self.variable.get_constant("limit_upper"))  != -1 or \
               l_descomponent_rules.find(self.variable.get_constant("reject_upper")) != -1:
                
                l_return_rules.append(l_descomponent_rules)
                
        return l_return_rules


    # Return status ufw
    def set_ufw_log(self, status_ufw_log):
        commands.getstatusoutput(self.variable.get_command(status_ufw_log))
        return status_ufw_log


    # Get status ufw
    def get_ufw_log(self):
        l_status = commands.getstatusoutput(self.variable.get_command("status_verbose"))
        if l_status[1].find("Logging: on (high)") != -1:
            return self.variable.get_constant("ufw_log_high")
        elif l_status[1].find("Logging: on (medium)") != -1:
            return self.variable.get_constant("ufw_log_medium")
        elif l_status[1].find("Logging: on (low)") != -1:
            return self.variable.get_constant("ufw_log_low")
        else:
            return self.variable.get_constant("ufw_log_off")


    # Return status ufw
    def set_gufw_log(self, status_gufw_log):
        if not self.variable.dev:
            l_status_gufw_log = status_gufw_log
        else:
            l_status_gufw_log = status_gufw_log + "_dev"
            
        commands.getstatusoutput(self.variable.get_command(l_status_gufw_log))
        return status_gufw_log


    # Get status ufw
    def get_gufw_log(self):
        if not self.variable.dev:
            l_command = commands.getstatusoutput(self.variable.get_command("cfg_gufw_log"))
        else:
            l_command = commands.getstatusoutput(self.variable.get_command("cfg_gufw_log_dev"))
            
        if l_command[0] == 0:
            return self.variable.get_constant("gufw_log_on")
        else:
            return self.variable.get_constant("gufw_log_off")


    # Add rule 
    def add_rule_component(self, service, action, protocol, fromip, fromport, toip, toport):
        # Component rule
        if service == self.variable.get_constant("service_no"):
            l_rule = "LANG=en ufw &action proto &protocol from &fromIP port &fromPort to &toIP port &toPort"
        else:
            l_rule = "LANG=en ufw &action &toPort"
        
        # Action
        l_rule = l_rule.replace("&action", action)
        
        # Protocol
        if protocol != self.variable.get_constant("both"):
            l_rule = l_rule.replace("&protocol", protocol)
        else:
            l_rule = l_rule.replace(" proto &protocol ", " ")
            
        # FROM
        if fromip != "":
            l_rule = l_rule.replace("&fromIP", fromip)
        else:
            l_rule = l_rule.replace("&fromIP", self.variable.get_constant("any"))
            
        if fromport != "":
            l_rule = l_rule.replace("&fromPort", fromport)
        else:
            l_rule = l_rule.replace(" port &fromPort ", " ")
            
        # TO
        if toip != "":
            l_rule = l_rule.replace("&toIP", toip)
        else:
            l_rule = l_rule.replace("&toIP", self.variable.get_constant("any"))
            
        if toport != "":
            l_rule = l_rule.replace("&toPort", toport)
        else:
            l_rule = l_rule.replace(" port &toPort", "")

        # Return rule command
        return l_rule


    # Command Add rule
    def add_rule(self, rule):
        l_result = commands.getstatusoutput(rule)
        return l_result[1]


    # Add rule 
    def remove_rule_component(self, rule_to, action, rule_from):
        # Component rule
        protocol = ""
        toip     = ""
        toport   = ""
        fromip   = "" 
        fromport = ""
        l_rule   = "LANG=en ufw delete &action proto &protocol from &fromIP port &fromPort to &toIP port &toPort"
        
        # Set protocol
        if rule_to.find("/tcp")   != -1 or rule_from.find("/tcp") != -1:
            protocol = self.variable.get_constant("tcp")
            rule_to   = rule_to.replace("/tcp","")
            rule_from = rule_from.replace("/tcp","")
        elif rule_to.find("/udp")   != -1 or rule_from.find("/udp") != -1:
            protocol = self.variable.get_constant("udp")
            rule_to   = rule_to.replace("/udp","")
            rule_from = rule_from.replace("/udp","")
        else:
            protocol = self.variable.get_constant("both")

        
        # Set IP/Port TO
        l_to_aux = rule_to.split(" ")
        if len(l_to_aux) == 1:
            # IP or Port?
            if l_to_aux[0] == self.variable.get_constant("anywhere"):
                toip = ""
            elif ( l_to_aux[0].find(".") != -1 ) or ( len(l_to_aux[0]) > 11 ):
                toip = l_to_aux[0]
            else:
                toport = l_to_aux[0]
        elif len(l_to_aux) == 2:
            toip   = l_to_aux[0]
            toport = l_to_aux[1]
        elif len(l_to_aux) == 3:
            # It's a app command
            l_rule = "LANG=en ufw delete " + l_to_aux[2]
            return l_rule
            
        # Set IP/Port FROM
        l_from_aux = rule_from.split(" ")
        if len(l_from_aux) == 1:
            # IP or Port?
            if l_from_aux[0] == self.variable.get_constant("anywhere"):
                fromip = ""
            elif ( l_from_aux[0].find(".") != -1 ) or ( len(l_from_aux[0]) > 11 ):
                fromip = l_from_aux[0]
            else:
                fromport = l_from_aux[0]
        elif len(l_from_aux) == 2:
            fromip   = l_from_aux[0]
            fromport = l_from_aux[1]
        elif len(l_from_aux) == 3:
            # It's a app command
            l_rule = "LANG=en ufw delete " + l_from_aux[2]
            return l_rule
            
        # Set rule    
        # Action
        l_rule = l_rule.replace("&action", action)
        
        # Protocol
        if protocol != self.variable.get_constant("both"):
            l_rule = l_rule.replace("&protocol", protocol)
        else:
            l_rule = l_rule.replace(" proto &protocol ", " ")
            
        # FROM
        if fromip != "":
            l_rule = l_rule.replace("&fromIP", fromip)
        else:
            l_rule = l_rule.replace("&fromIP", self.variable.get_constant("any"))
            
        if fromport != "":
            l_rule = l_rule.replace("&fromPort", fromport)
        else:
            l_rule = l_rule.replace(" port &fromPort ", " ")
            
        # TO
        if toip != "":
            l_rule = l_rule.replace("&toIP", toip)
        else:
            l_rule = l_rule.replace("&toIP", self.variable.get_constant("any"))
            
        if toport != "":
            l_rule = l_rule.replace("&toPort", toport)
        else:
            l_rule = l_rule.replace(" port &toPort", "")

        # Return rule command
        return l_rule


    # Command Add rule
    def remove_rule(self, rule):
        l_result = commands.getstatusoutput(rule)
        return l_result[1]
